package GUT;

import GUT.quals.*;
import checkers.basetype.BaseTypeChecker;
import checkers.quals.TypeQualifiers;
import checkers.source.SupportedLintOptions;
import checkers.types.AnnotatedTypeMirror;
import checkers.types.AnnotatedTypeMirror.AnnotatedDeclaredType;
import checkers.util.AnnotationUtils;
import checkers.util.TypesUtils;

import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.SupportedOptions;
import javax.lang.model.element.AnnotationMirror;


/**
 * The main checker class for the Generic Universe Types checker.
 *
 * @author wmdietl
 */
/*
 * Use this for warning messages:
 * messager.printMessage(javax.tools.Diagnostic.Kind.WARNING, "message");
 */
@TypeQualifiers({ Any.class, Peer.class, Rep.class,
    Lost.class, Self.class, Bottom.class})
@SupportedOptions( { "warn" } )
@SupportedLintOptions({"allowLost", "checkOaM", "checkStrictPurity"})
public class GUTChecker extends BaseTypeChecker {

    // Only public so that the GUTI package can access these fields.
    // Maybe merge the packages again?
    // TODO: change to getters?
    public AnnotationMirror ANY, PEER, REP, LOST, SELF, BOTTOM, PURE;

	public boolean checkOaM;
	public boolean checkStrictPurity;
	public boolean allowLost;

	public boolean warn_staticpeer;



    @Override
    public void initChecker(ProcessingEnvironment env) {
        AnnotationUtils annoFactory = AnnotationUtils.getInstance(env);
        ANY = annoFactory.fromClass(Any.class);
        PEER = annoFactory.fromClass(Peer.class);
        REP = annoFactory.fromClass(Rep.class);
        LOST = annoFactory.fromClass(Lost.class);
        SELF = annoFactory.fromClass(Self.class);
        BOTTOM = annoFactory.fromClass(Bottom.class);
        PURE = annoFactory.fromClass(Pure.class);

        super.initChecker(env);

        GUTQualifierUtils.setGUTChecker(this);

        this.allowLost = getLintOption("allowLost", false);
        this.checkOaM = getLintOption("checkOaM", false);
        this.checkStrictPurity = getLintOption("checkStrictPurity", false);

        String warn = env.getOptions().get("warn");
        if (warn==null) {
        	warn="";
        }
        warn_staticpeer = warn.contains("staticpeer");
    }

	public static boolean isAnyDefault(AnnotatedTypeMirror type) {
		// if (!(type instanceof AnnotatedDeclaredType))
			return false;
			/*
		DeclaredType dtype = ((AnnotatedDeclaredType)type).getUnderlyingType();
		return TypesUtils.isDeclaredOfName(dtype, "java.lang.String") ||
   			 TypesUtils.isDeclaredOfName(dtype, "java.lang.Character");
   			 */
	}

    /* TODO: purity/OaM checking
    @Override
    public boolean isAssignable(AnnotatedTypeMirror varType,
            AnnotatedTypeMirror receiverType, Tree varTree) {
        return super.isAssignable(varType, receiverType, varTree);
    }
    */

    /*
     * The supermethod is currently OK, it only checks for a subtype relationship between the erased types.
     * Depends on what we use as default modifier. Super meth is ok for Any default, but not for peer.
     */
    @Override
    public boolean isValidUse(AnnotatedDeclaredType elemType, AnnotatedDeclaredType use) {
    	return true;
    	// return super.isValidUse(elemType, use);
    }

    @Override
    public boolean isSubtype(AnnotatedTypeMirror sub, AnnotatedTypeMirror sup) {
    	if (sub.getUnderlyingType().getKind().isPrimitive() ||
    		sup.getUnderlyingType().getKind().isPrimitive() ) {
    		// Ignore boxing/unboxing
    		return true;
    	}
    	if (TypesUtils.isString(sub.getUnderlyingType()) ||
    	    TypesUtils.isString(sup.getUnderlyingType())) {
    	    return true;
    	}
    	
    	// System.out.println("Subtype: " + sub);
    	// System.out.println("Supertype: " + sup);
        return super.isSubtype(sub, sup);
    }

    // **********************************************************************
    // Factory methods
    // **********************************************************************
    /*
    @Override
    protected QualifierHierarchy createQualifierHierarchy() {
        return new GUTQualifierHierarchy((GraphQualifierHierarchy) super.createQualifierHierarchy());
    }

    @Override
    protected TypeHierarchy createTypeHierarchy() {
        return new GUTTypeHierarchy(getQualifierHierarchy());
    }

    private final class GUTQualifierHierarchy extends GraphQualifierHierarchy {
        public GUTQualifierHierarchy(GraphQualifierHierarchy hierarchy) {
            super(hierarchy);
        }

        @Override
        public boolean isSubtype(AnnotationMirror rhs, AnnotationMirror lhs) {
            return rhs == null || lhs == null
                   || super.isSubtype(rhs, lhs);
        }
    }

    private final class GUTTypeHierarchy extends TypeHierarchy {
        public GUTTypeHierarchy(QualifierHierarchy qualifierHierarchy) {
            super(qualifierHierarchy);
        }

        @Override
        protected boolean isSubtypeAsTypeArgument(AnnotatedTypeMirror rhs, AnnotatedTypeMirror lhs) {
            return super.isSubtypeAsTypeArgument(rhs, lhs);
        }

        @Override
        protected boolean isSubtypeTypeArguments(AnnotatedDeclaredType rhs, AnnotatedDeclaredType lhs) {
            return super.isSubtypeTypeArguments(rhs, lhs);
        }
    }
    */
}